//+------------------------------------------------------------------+
//|                                          StochasticExpansion.mq5 |
//|                                  Copyright  2010, Ivan Kornilov |
//|                                     StochasticExpansion_v1.1.mq4 |
//+------------------------------------------------------------------+
//|      SmoothAlgorithms.mqh                 |
//|     (): MetaTrader\MQL5\Include   |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, Ivan Kornilov."
#property link "excelf@gmail.com"
#property version   "1.10"
//----     
#property indicator_separate_window
//----       2 
#property indicator_buffers 2
//----     
#property indicator_plots   1
//----     
#property indicator_type1   DRAW_COLOR_LINE
//----      
#property indicator_color1  Gray,Magenta,DodgerBlue
//----   -  
#property indicator_style1  STYLE_SOLID
//----     2
#property indicator_width1  2
//----   
#property indicator_label1  "StochasticExpansion"
//+-----------------------------------+
//|    CXMA             |
//+-----------------------------------+
#include <SmoothAlgorithms.mqh> 
//+-----------------------------------+
//----    CXMA   SmoothAlgorithms.mqh
CXMA XMA1;
//+-----------------------------------+
//|             |
//+-----------------------------------+
/*enum Smooth_Method -     SmoothAlgorithms.mqh
  {
   MODE_SMA_,  //SMA
   MODE_EMA_,  //EMA
   MODE_SMMA_, //SMMA
   MODE_LWMA_, //LWMA
   MODE_JJMA,  //JJMA
   MODE_JurX,  //JurX
   MODE_ParMA, //ParMA
   MODE_T3,    //T3
   MODE_VIDYA, //VIDYA
   MODE_AMA,   //AMA
  }; */
//+-----------------------------------+
//|         |
//+-----------------------------------+
input int KPeriod = 13;
input int Slowing = 6;
input int NoiseFilter=6;
input Smooth_Method Filter_Method=MODE_SMA; //                     
input int Filter_Phase=15;                  //  
input int Shift=0;                          //     
//+-----------------------------------+
//----   ,   
//----      
double ExtLineBuffer[];
double ColorExtLineBuffer[];
//----   ,   
//----      
int Count[];
double Lowest[],Highest[];
//----      
int min_rates_total,min_rates_total_1;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+ 
void OnInit()
  {
//----     
   min_rates_total_1=KPeriod+Slowing;
   min_rates_total=min_rates_total_1+XMA1.GetStartBars(Filter_Method,NoiseFilter,Filter_Phase)+1;
//----       
   XMA1.XMALengthCheck("KPeriod", KPeriod);
   XMA1.XMALengthCheck("Slowing", Slowing);
   XMA1.XMALengthCheck("NoiseFilter", NoiseFilter);
   XMA1.XMAPhaseCheck("Filter_Phase", Filter_Phase, Filter_Method);

//----     
   if(ArrayResize(Count,Slowing)<Slowing) Print("      Count[]");
   if(ArrayResize(Lowest,Slowing)<Slowing) Print("      Lowest[]");
   if(ArrayResize(Highest,Slowing)<Slowing) Print("      Highest[]");

//----    ExtLineBuffer[]   
   SetIndexBuffer(0,ExtLineBuffer,INDICATOR_DATA);
//----       Shift
   PlotIndexSetInteger(0,PLOT_SHIFT,Shift);
//----      
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,min_rates_total);
//----      
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,EMPTY_VALUE);
//----    ,   
   ArraySetAsSeries(ExtLineBuffer,true);

//----    ColorExtLineBuffer[]       
   SetIndexBuffer(1,ColorExtLineBuffer,INDICATOR_COLOR_INDEX);
//----      
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,min_rates_total);
//----    ,   
   ArraySetAsSeries(ColorExtLineBuffer,true);

//----      
   string shortname;
   StringConcatenate(shortname,"StochasticExpansion(",KPeriod," ,",Slowing," ,",NoiseFilter," ,",Shift,")");
//----           
   IndicatorSetString(INDICATOR_SHORTNAME,shortname);
//----     
   IndicatorSetInteger(INDICATOR_DIGITS,2);
//----
  }
//+------------------------------------------------------------------+
//|                       |
//+------------------------------------------------------------------+   
void Recount_ArrayZeroPos(int &CountArray[],//        
                          int Size)
  {
//----
   int numb,Max1,Max2;
   static int count=1;

   Max2=Size;
   Max1=Max2-1;

   count--;
   if(count<0) count=Max1;

   for(int iii=0; iii<Max2; iii++)
     {
      numb=iii+count;
      if(numb>Max1) numb-=Max2;
      CountArray[iii]=numb;
     }
//----
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,    //       
                const int prev_calculated,//       
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//----       
   if(rates_total<min_rates_total) return(0);

//----    
   int limit,bar,maxbar;
   double KLine1=0,KLine2=0,KLine,value;

//----    limit    
   if(prev_calculated>rates_total || prev_calculated<=0) //      
     {
      limit=rates_total-min_rates_total_1-1; //      
     }
   else
     {
      limit=rates_total-prev_calculated;     //      
     }

//----    ,     
   ArraySetAsSeries(low,true);
   ArraySetAsSeries(high,true);
   ArraySetAsSeries(close,true);

//----  
   maxbar=rates_total-min_rates_total_1-1;

//----    
   for(bar=limit; bar>=0 && !IsStopped(); bar--)
     {
      Lowest[Count[0]]=low[ArrayMinimum(low,bar,KPeriod)];
      Highest[Count[0]]=high[ArrayMaximum(high,bar,KPeriod)];

      if(bar>maxbar)
        {
         ExtLineBuffer[bar]=EMPTY_VALUE;
         ColorExtLineBuffer[bar]=0;
         Recount_ArrayZeroPos(Count,Slowing);
         continue;
        }

      double sumlow=0;
      double sumhigh=0;

      for(int k=0; k<Slowing; k++)
        {
         int bark=bar+k;
         int r=Count[k];
         sumlow += close[bark] - Lowest[r];
         sumhigh+= Highest[r] - close[bark];
        }

      if(sumhigh) KLine1=+sumlow/sumhigh;
      if(sumlow)  KLine2=-sumhigh/sumlow;
      KLine=KLine1+KLine2;
      ExtLineBuffer[bar]=KLine;
      value=XMA1.XMASeries(maxbar,prev_calculated,rates_total,Filter_Method,Filter_Phase,NoiseFilter,KLine,bar,true);

      if(KLine<value) ColorExtLineBuffer[bar]=1;
      else if(KLine>value) ColorExtLineBuffer[bar]=2;
      else ColorExtLineBuffer[bar]=0;

      if(bar) Recount_ArrayZeroPos(Count,Slowing);
     }
//----     
   return(rates_total);
  }
//+------------------------------------------------------------------+
